example_tilesets
pleat.example_tilesets ¶
Predefined tile sets for Archimedean, Platonic, and curved tilings.
align_tiles ¶
Set mutual gluing instructions between two tiles along the given edge labels.
Source code in pleat/example_tilesets.py
platonic ¶
Return tiles for the Euclidean Platonic tiling with n-gons (n must be 3, 4, or 6).
Source code in pleat/example_tilesets.py
square_strip ¶
Return tiles for a strip of squares with two distinct edge labels.
t_3_12_12 ¶
Return tiles for the (3.12.12) Archimedean tiling.
Source code in pleat/example_tilesets.py
t_4_6_12 ¶
Return tiles for the (4.6.12) Archimedean tiling.
Source code in pleat/example_tilesets.py
t_3_3_4_3_4 ¶
Return tiles for the (3.3.4.3.4) Archimedean tiling.
Source code in pleat/example_tilesets.py
t_3_3_3_3_6 ¶
Return tiles for the (3.3.3.3.6) Archimedean tiling.
Source code in pleat/example_tilesets.py
pgg_2x ¶
Return tiles for a pgg wallpaper group tiling with two copies.
Source code in pleat/example_tilesets.py
u2_4_6_12__3_4_6_4 ¶
Return tiles for the 2-uniform (4.6.12; 3.4.6.4) tiling.
Source code in pleat/example_tilesets.py
archimedean_vertex_to_geometry ¶
Determine the geometry (Euclidean, spherical, or hyperbolic) from vertex face orders.
Parameters:
| Name | Type | Description | Default |
|---|---|---|---|
face_orders
|
'list[int] | tuple[int, ...]'
|
Sizes of the faces meeting at a vertex. |
required |
Returns:
| Type | Description |
|---|---|
type
|
class: |
type
|
class: |
type
|
angle sum exactly |
Source code in pleat/example_tilesets.py
curved_platonic ¶
Return tiles for a {n, k} Platonic tiling in the appropriate geometry.
Source code in pleat/example_tilesets.py
curved_snub ¶
Return tiles for the snub {n, k} tiling in the appropriate geometry.
Source code in pleat/example_tilesets.py
curved_expand ¶
Return tiles for the expanded {n, k} tiling in the appropriate geometry.
Source code in pleat/example_tilesets.py
curved_truncate ¶
Return tiles for the truncated {n, k} tiling in the appropriate geometry.
Source code in pleat/example_tilesets.py
curved_ambo ¶
Return tiles for the rectified (ambo) {n, k} tiling in the appropriate geometry.
Source code in pleat/example_tilesets.py
curved_omnitruncate ¶
Return tiles for the omnitruncated {n, k} tiling in the appropriate geometry.
Source code in pleat/example_tilesets.py
curved_zip ¶
Return tiles for the zipped (bitruncated dual) {n, k} tiling in the appropriate geometry.